@@ -17,44 +17,6 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* Some info gathered about the registers:
-00 - 1st Digit
-01 - 2nd
-02 - 3rd
-03 - 4th
-04 - 5th
-05 - 6th
-
-06 - Colon 1, 2, 3 (bottom, middle, both)
-07 - Not sure... 
-08 - Show play symbol
-09 - Show pause symbol
-10 - 
-11 - HD Mode (HD Symbol)
-12 - CD Mode (CD Symbol)
-13 - FM Mode (FM + Radio on)
-14 - AM Mode (AM + Radio on)
-15 - Turns off Radio / Current mode? (Doesn't clear icon)
-
-30 - Hour (Hex e.g. 0x23 = 11pm)
-31 - Minute
-
-33 - Reads back the value written...
-34 - Displays temperature (hex value written). 
-35 - Go back to digit mode (resets to -- --:--)
-
-36 - Volume control count (8-bit +/-)
-37 - 
-38 - 
-39 - 
-40 -
-
-41 - Display time - how to set?!!
-
-Don't go above 86 - the I2C bus freezes! 
-*/
-
-
 #include <sys/io.h>
 #include <errno.h>
 #include <string.h>
@@ -65,90 +27,14 @@ Don't go above 86 - the I2C bus freezes!
 #include <getopt.h>
 #include <time.h>
 #include "i2c-dev.h"
-
-#define DM_VERSION "00.03"
-
-// Address on the I2C bus of the AudioDL (OZ268) IC. 
-#define AUDIODJ_I2C_ADDRESS 0x49
-
-// Update intervals for temperature display and volume control in milli-seconds. 
-#define TEMPERATURE_UPDATE_INTERVAL_MS 2000
-#define VOLUME_UPDATE_INTERVAL_MS 50
-
-#define I2C_TRANSACTION_DELAY_MS 1000
-
-void print_i2c_busses(int);
-void help(char *argv0) __attribute__ ((noreturn));
-
-#ifdef FALSE
-#define BOOL bool
-#else
-typedef enum _bool
-{
-    FALSE=0,
-    TRUE
-} BOOL;
-#endif
-
-    // First digit: 
-    // 0x80 of 0x00 makes LED not light up for representative mode. 
-    
-    // All digits:
-    // 0x0f = blank
-    // 0x0e = 'C' (2nd Digit = 'D') CD MODE LED
-    // 0x0d = 'P' (2nd Digit = '-') HD MODE LED
-    // 0x0c = '-' (2nd Digit = 'M') Invalid MODE
-    // 0x0b = 'A'                   AM MODE LED 
-    // 0x0a = 'F'                   FM MODE LED
-    // < 0x0a = Number.i
-
-typedef enum _radio_mode
-{
-    OFF_MODE = 0,
-    AM_RADIO,
-    FM_RADIO,
-    CD_MODE,
-    HD_MODE
-} RADIO_MODE;
-
-typedef enum _leds
-{
-    LED_NONE = 0x8f,
-    LED_AM = 0x0b,
-    LED_FM = 0x0a,
-    LED_CD = 0x0e,
-    LED_HD = 0x0d,
-} LED;
-
-
-// Writing to Reg 0x07 does nothing, 0x08 = PLAY LED, 0x09 = PAUSE LED.
-typedef enum _pp
-{
-    PP_NONE = 0x07,
-    PP_PLAY = 0x08,
-    PP_PAUSE = 0x09
-} PP;
-
-
-// Dots are in reg 0x06
-    // 0x00 = No Dots.
-    // 0x01 = middle dot
-    // 0x02 = bottom dot 
-    // 0x03 = both dots
-typedef enum _dots
-{
-    DOTS_NONE = 0,
-    DOTS_MIDDLE,
-    DOTS_BOTTOM,
-    DOTS_BOTH
-} DOTS;
-
+#include "setpanel.h"
 
 BOOL bI2CInitialised = FALSE;
 
 static struct option longopts[] = {
     { "i2c",         1, 0, 'i' },
     { "radio",       0, 0, 'r' },
+    { "control",     1, 0, 'c' },
     { "display",     1, 0, 'd' },
     { "temp",        0, 0, 't' },
     { "update-temp", 0, 0, 'u' },
@@ -160,7 +46,7 @@ static struct option longopts[] = {
     { NULL,          0, 0,  0  }
 };
 
-void help(char *argv0)
+static void help(char *argv0)
 {
         printf(
 "\n"
@@ -172,6 +58,15 @@ void help(char *argv0)
 "  -i, --i2c <n>              Sets the I2C Bus (Compulsary)\n"
 "  -r, --radio [am | fm | off]  Set the radio to am, fm or off\n"
 "\n"
+"  -c, --control [seek- | seek+ | fine- | fine+ | ch- | ch+ | prog | cancel | store]\n"
+"                             Controls the radio\n"
+"                             seek+/- seeks the next station\n"
+"                             fine+/- manual increases / decreases frequency\n"
+"                             ch+/- selects the stored presets\n"
+"                             prog begins storing the current frequency (use ch+/- to choose)\n"
+"                             cancel aborts the store\n"
+"                             store commits the frequency to the flashing preset\n"
+"\n"
 "  -d, --display [\"xxxxxx\"]   Set the display to the numbers specified\n"
 "                             Available are: [0-9, A, C, F, P]\n"
 "                             Limited availability: [M, D, -] (try them :)\n"
@@ -201,7 +96,7 @@ void help(char *argv0)
   exit(1);
 }
 
-int digimatrix_write_byte_data(int file, char reg, char data)
+static int digimatrix_write_byte_data(int file, char reg, char data)
 {
     int res;
     // TODO: Implement kernel specific stuff here. 
@@ -210,7 +105,7 @@ int digimatrix_write_byte_data(int file, char reg, char data)
     return res;
 }
 
-int digimatrix_write_byte(int file, char data)
+static int digimatrix_write_byte(int file, char data)
 {
     int res;
     // TODO: Implement kernel specific stuff here. 
@@ -219,7 +114,7 @@ int digimatrix_write_byte(int file, char data)
     return res;
 }
 
-int digimatrix_read_byte(int file)
+static int digimatrix_read_byte(int file)
 {
     int res;
     // TODO: Implement kernel specific stuff here. 
@@ -228,7 +123,48 @@ int digimatrix_read_byte(int file)
     return res;
 }
 
-static void digimatrix_TurnOnRadio(int file, RADIO_MODE radio_mode)
+void digimatrix_Control(int file, CONTROL control) { 
+ 
+    switch (control) { 
+        case SEEK_DOWN: 
+            digimatrix_write_byte_data(file, 0x27, 0x00); 
+            break; 
+ 
+        case SEEK_UP: 
+            digimatrix_write_byte_data(file, 0x13, 0x00); 
+            break; 
+ 
+        case FINE_TUNE_DOWN: 
+            digimatrix_write_byte_data(file, 0x26, 0x00); 
+            break; 
+ 
+        case FINE_TUNE_UP: 
+            digimatrix_write_byte_data(file, 0x25, 0x00); 
+            break; 
+ 
+        case CHANNEL_PREV: 
+            digimatrix_write_byte_data(file, 0x11, 0x00); 
+            break; 
+ 
+        case CHANNEL_NEXT: 
+            digimatrix_write_byte_data(file, 0x10, 0x00); 
+            break; 
+ 
+        case STORE_START: 
+            digimatrix_write_byte_data(file, 0x15, 0x00); 
+            break; 
+ 
+        case STORE_CANCEL: 
+            digimatrix_write_byte_data(file, 0x16, 0x00); 
+            break; 
+        
+	case STORE_COMMIT: 
+            digimatrix_write_byte_data(file, 0x19, 0x00); 
+            break; 
+    } 
+} 
+ 
+void digimatrix_TurnOnRadio(int file, RADIO_MODE radio_mode)
 {
     digimatrix_write_byte_data(file, 0x23, 0x02);
 
@@ -267,8 +203,7 @@ static void digimatrix_TurnOnRadio(int file, RADIO_MODE radio_mode)
     }
 }
 
-static void digimatrix_DisplayNumbers(int file, char digits[6], 
-                                      LED led, PP pp, DOTS dots)
+void digimatrix_DisplayNumbers(int file, char digits[6], LED led, PP pp, DOTS dots)
 {
     int n;
     
@@ -344,7 +279,7 @@ static void digimatrix_DisplayNumbers(int file, char digits[6],
     
 }
 
-static void digimatrix_DisplayTemp(int file, int temp)
+void digimatrix_DisplayTemp(int file, int temp)
 {
     char temp_dec = 0;
     // Get the temperature from the IT87
@@ -356,7 +291,7 @@ static void digimatrix_DisplayTemp(int file, int temp)
     }
     else
     {
-        outb(0x29, 0x295);
+        outb(IT8705_TEMP_REG, 0x295);
         temp = inb(0x296);
     }
 
@@ -374,7 +309,7 @@ static void digimatrix_DisplayTemp(int file, int temp)
     //digimatrix_write_byte_data(file, 0x0f, 0x01);
 }
 
-static void digimatrix_UpdateTemp(int file, int temp)
+void digimatrix_UpdateTemp(int file, int temp)
 {
     char temp_dec = 0;
     // Get the temperature from the IT87
@@ -386,7 +321,7 @@ static void digimatrix_UpdateTemp(int file, int temp)
     }
     else
     {
-        outb(0x29, 0x295);
+        outb(IT8705_TEMP_REG, 0x295);
         temp = inb(0x296);
     }
 
@@ -397,7 +332,7 @@ static void digimatrix_UpdateTemp(int file, int temp)
     digimatrix_write_byte_data(file, 0x22, temp_dec);
 }
 
-int digimatrix_I2CInit(int i2cbus)
+static int digimatrix_I2CInit(int i2cbus)
 {
   int address = AUDIODJ_I2C_ADDRESS;
   int e1, e2, e3, file;
@@ -517,7 +452,7 @@ int main(int argc, char *argv[])
 */
     for (;;) 
     {
-        if ((ch = getopt_long(argc, argv, "i:r:d:tulmsvh", longopts, &longindex)) == -1)
+        if ((ch = getopt_long(argc, argv, "i:r:d:c:tulmsvh", longopts, &longindex)) == -1) 
         {
             break;
         }
@@ -557,6 +492,47 @@ int main(int argc, char *argv[])
                     digimatrix_TurnOnRadio(file, OFF_MODE);            
                 }
             break;
+
+        case 'c': 
+                printf("Controlling Radio: "); 
+                if (!strcmp(optarg, "seek-")) { 
+                    printf("Seeking down\n"); 
+                    digimatrix_Control(file, SEEK_DOWN); 
+                } 
+                else if (!strcmp(optarg, "seek+")) { 
+                    printf("Seeking up\n"); 
+                    digimatrix_Control(file, SEEK_UP); 
+                } 
+                else if (!strcmp(optarg, "fine-")) { 
+                    printf("Fine tuning down\n"); 
+                    digimatrix_Control(file, FINE_TUNE_DOWN); 
+                } 
+                else if (!strcmp(optarg, "fine+")) { 
+                    printf("Fine tuning up\n"); 
+                    digimatrix_Control(file, FINE_TUNE_UP); 
+                } 
+                else if (!strcmp(optarg, "ch-")) { 
+                    printf("Previous preset\n"); 
+                    digimatrix_Control(file, CHANNEL_PREV); 
+                } 
+                else if (!strcmp(optarg, "ch+")) { 
+                    printf("Next preset\n"); 
+                    digimatrix_Control(file, CHANNEL_NEXT); 
+                } 
+                else if (!strcmp(optarg, "prog")) { 
+                    printf("Program preset - now select preset with ch+/-\n"); 
+                    digimatrix_Control(file, STORE_START); 
+                } 
+                else if (!strcmp(optarg, "cancel")) { 
+                    printf("Program cancelled\n"); 
+                    digimatrix_Control(file, STORE_CANCEL); 
+                } 
+                else if (!strcmp(optarg, "store")) { 
+                    printf("Preset stored\n"); 
+                    digimatrix_Control(file, STORE_COMMIT); 
+                } 
+            break; 
+ 
             
         case 'd':
 	    {
